local NAME, addon = ...
addon.NAME = NAME
XLootMaster = addon
local L = addon.L

local workframe=CreateFrame("Frame")
workframe:SetScript("OnEvent", 
function(self,event,...) 
	return self[event] and self[event](...)
end)

local RAID_CLASS_COLORS = CUSTOM_CLASS_COLORS or _G.RAID_CLASS_COLORS
local opt, LD

addon.defaults = {
	menu_roll = true,
	menu_disenchant = true,
	menu_bank = true,
	menu_self = true,
}
local me = UnitName('player')
local banker_name, disenchanter_name
local my_index, banker_index, disenchanter_index
local classesInRaid, class_players = {}, {}
local player_indices, index_name = {}, {}
local randoms = {}
local classes_english = {}
local candidate, color, lclass, className, slot, info

for i,class in ipairs(CLASS_SORT_ORDER) do
	classes_english[class] = true
end
local wipe, match = table.wipe, string.match

local hexColors = {}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = "|c" .. v.colorStr
end
hexColors["UNKNOWN"] = string.format("|cff%02x%02x%02x", 0.6*255, 0.6*255, 0.6*255)

if CUSTOM_CLASS_COLORS then
	local function update()
		for k, v in pairs(CUSTOM_CLASS_COLORS) do
			hexColors[k] = "|c" .. v.colorStr
		end
	end
	CUSTOM_CLASS_COLORS:RegisterCallback(update)
	update()
end

-- Utility functions
local function printall(...)
	if DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage(string.join(", ",tostringall(...)))
	end
end
addon.printall = printall

local function dump(val)
	UIParentLoadAddOn("Blizzard_DebugTools")
	_G['xlminfostruct'] = val
	DevTools_DumpCommand('xlminfostruct')
	_G['xlminfostruct'] = nil
end
addon.dump = dump

local function groupOutChannel()
	local out
	if IsInRaid() then
		if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") or IsEveryoneAssistant() then
			out = "RAID_WARNING"
		else
			out = "RAID"
		end
	elseif IsInGroup() then
		out = "PARTY"
	end
	return out
end

-- Addon functions
local function XLootMaster_GiveLoot(frame)
	-- this is where we'll be doing our distribution announcing according to options
	
	if ( LootFrame.selectedQuality >= MASTER_LOOT_THREHOLD ) then
		StaticPopup_Show("CONFIRM_LOOT_DISTRIBUTION", ITEM_QUALITY_COLORS[LootFrame.selectedQuality].hex..LootFrame.selectedItemName..FONT_COLOR_CODE_CLOSE, frame.text, "LootWindow");
	else
		GiveMasterLoot(LootFrame.selectedSlot, frame.value)
	end
	CloseDropDownMenus()
end

local function XLootMaster_SpawnRoll(frame)
	DoMasterLootRoll(frame.value)
end

local function XLootMaster_RaidRoll(frame, players)
	-- todo: announce player:id list
	local to = #players
	local out = groupOutChannel()
	if to >=1 then
		workframe:RegisterEvent("CHAT_MSG_SYSTEM")
		if out then
			SendChatMessage("Raid Roll:"..GetLootSlotLink(LootFrame.selectedSlot),out) -- localize this
			local k,names = 1,""
			for i=1,to do
				names = (k==1) and (i..":"..index_name[i]) or (names..", "..i..":"..index_name[i])
				if i==to or k==3 then
					SendChatMessage(names,out)
					names = ""
				end
				k = k<3 and k+1 or 1
			end
		end
 		RandomRoll(1,to)
 	end
end
local rollCapture = gsub(gsub(gsub(RANDOM_ROLL_RESULT, "[()]", "."), "%%s", "([^ ]+)") , "%%d", "(%%d+)")
function workframe.CHAT_MSG_SYSTEM(...)
	local who, roll, from, to = match(..., rollCapture)
	if who == me then
		workframe.value = roll
		workframe:UnregisterEvent("CHAT_MSG_SYSTEM")
		XLootMaster_GiveLoot(workframe)
	end
end

function XLootMaster.AddMenuTitle(title)
	info.isTitle = nil
	info.text = title
	info.textHeight = 12
	info.notCheckable = 1
	info.disabled = 1
	info.disablecolor = YELLOW_FONT_COLOR_CODE
	info.notClickable = 1
	UIDropDownMenu_AddButton(info)
	info.notClickable = nil
	info.disablecolor = nil
end

function XLootMaster.AddMenuSeparator()
	info.disabled = 1
  info.text = ""
  info.hasArrow = nil
  UIDropDownMenu_AddButton(info, level)
end

function XLootMaster.BuildPartyMenu(level)
	-- In a party
	if level == 1 then
		for i=1, MAX_PARTY_MEMBERS+1, 1 do
			candidate,lclass,className = GetMasterLootCandidate(slot,i)
			if candidate then
				-- Add candidate button
				info.text = candidate
				info.colorCode = hexColors[className] or hexColors["UNKNOWN"]
				info.textHeight = 12
				info.value = i
				info.notCheckable = 1
				info.hasArrow = nil
				info.isTitle = nil
				info.disabled = nil
				info.func = XLootMaster_GiveLoot
				UIDropDownMenu_AddButton(info)
			end
		end
	end
end

function XLootMaster.BuildRaidMenuRecipients(level)
	if level == 1 then
		if (my_index and opt.menu_self) or (banker_index and opt.menu_bank) or (disenchanter_index and opt.menu_disenchant) then
			-- XLootMaster.AddMenuSeparator()
	    info.disabled = nil
			info.isTitle = nil
			info.text = L.RECIPIENTS
			info.colorCode = YELLOW_FONT_COLOR_CODE
			info.textHeight = 12
			info.hasArrow = 1
			info.notCheckable = 1
			info.value = "RECIPIENTS"
			info.func = nil
			info.disabled = nil
			UIDropDownMenu_AddButton(info)
		end
	elseif level == 2 then
		if UIDROPDOWNMENU_MENU_VALUE == "RECIPIENTS" then -- special recipients submenu
			if my_index then
				candidate,lclass,className = GetMasterLootCandidate(slot,my_index)
				if candidate and candidate == me then
					info.colorCode = hexColors[className] or hexColors["UNKNOWN"]
					info.isTitle = nil
					info.textHeight = 12
					info.value = my_index
					info.notCheckable = 1
					info.text = L.ML_SELF
					info.func = XLootMaster_GiveLoot
					info.icon = "Interface\\GossipFrame\\VendorGossipIcon"
					UIDropDownMenu_AddButton(info,level)
				end
			end
			if banker_index then
				candidate,lclass,className = GetMasterLootCandidate(slot,banker_index)
				if candidate and candidate == banker_name then
					info.colorCode = "|cffffffff"
					info.isTitle = nil
					info.textHeight = 12
					info.value = banker_index
					info.notCheckable = 1
					info.text = L.ML_BANKER
					info.func = XLootMaster_GiveLoot
					info.icon = "Interface\\Minimap\\Tracking\\Banker"
					UIDropDownMenu_AddButton(info,level)
				end
			end
			if disenchanter_index then
				candidate,lclass,className = GetMasterLootCandidate(slot,disenchanter_index)
				if candidate and candidate == disenchanter_name then
					info.colorCode = "|cffffffff"
					info.isTitle = nil
					info.textHeight = 12
					info.value = disenchanter_index
					info.notCheckable = 1
					info.text = L.ML_DISENCHANTER
					info.func = XLootMaster_GiveLoot
					info.icon = "Interface\\Buttons\\UI-GroupLoot-DE-Up"
					UIDropDownMenu_AddButton(info,level)
				end
			end
		end
	end
end

function XLootMaster.BuildRaidMenuSpecialRolls(level)
	if level == 1 then
		info.isTitle = nil
		info.text = L.SPECIALROLLS
		info.colorCode = YELLOW_FONT_COLOR_CODE
		info.textHeight = 12
		info.hasArrow = 1
		info.notCheckable = 1
		info.value = "SPECIALROLLS"
		info.func = nil
		info.disabled = nil
		UIDropDownMenu_AddButton(info)		
	elseif level == 2 then
		if UIDROPDOWNMENU_MENU_VALUE == "SPECIALROLLS" then -- special rolls submenu
			info.colorCode = "|cffffffff"
			info.isTitle = nil
			info.textHeight = 12
			info.value = slot
			info.notCheckable = 1
			info.hasArrow = nil
			info.text = REQUEST_ROLL
			info.func = XLootMaster_SpawnRoll
			info.icon = "Interface\\Buttons\\UI-GroupLoot-Dice-Up"
			UIDropDownMenu_AddButton(info,level)
			
			if next(randoms) and opt.menu_roll then
				info.colorCode = "|cffffffff"
				info.isTitle = nil
				info.textHeight = 12
				info.value = randoms[math.random(1, #randoms)]
				info.notCheckable = 1
				info.text = L.ML_RANDOM
				info.func = XLootMaster_RaidRoll
 				info.arg1 = randoms
				info.icon = "Interface\\Buttons\\UI-GroupLoot-Coin-Up"
				UIDropDownMenu_AddButton(info,level)
			end
		end		
	end
end

function XLootMaster.BuildRaidMenu(level)
	-- In a raid
	if level == 1 then
		wipe(player_indices)
		wipe(index_name)
		wipe(classesInRaid)
		wipe(class_players)
		wipe(randoms)
		my_index, banker_index, disenchanter_index = nil,nil,nil
		for i = 1, MAX_RAID_MEMBERS do
			candidate,lclass,className = GetMasterLootCandidate(slot,i)
			if candidate then
				classesInRaid[className] = lclass
				table.insert(randoms, i)
				if candidate == me then
					my_index = i
				end
				if candidate == banker_name then
					banker_index = i
				end
				if candidate == disenchanter_name then
					disenchanter_index = i
				end
				player_indices[candidate] = i
				index_name[i] = candidate
				if not class_players[className] then class_players[className] = {} end
				table.insert(class_players[className],candidate)
			end
		end
		for i, class in ipairs(CLASS_SORT_ORDER) do
			local cname = classesInRaid[class]
			if cname then
				info.isTitle = nil
				info.text = cname
				info.colorCode = hexColors[class] or hexColors["UNKNOWN"]
				info.textHeight = 12
				info.hasArrow = 1
				info.notCheckable = 1
				info.value = class
				info.func = nil
				info.disabled = nil
				UIDropDownMenu_AddButton(info)
			end
		end
		
		XLootMaster.BuildRaidMenuRecipients(level)
	
	elseif level == 2 then
		-- raid class menu
		if classes_english[UIDROPDOWNMENU_MENU_VALUE] then -- classes submenus
			if next(class_players[UIDROPDOWNMENU_MENU_VALUE]) then
				table.sort(class_players[UIDROPDOWNMENU_MENU_VALUE])
				for _,cand in ipairs(class_players[UIDROPDOWNMENU_MENU_VALUE]) do
					-- Add candidate button
					info.text = cand
					info.colorCode = hexColors[UIDROPDOWNMENU_MENU_VALUE] or hexColors["UNKNOWN"]
					info.textHeight = 12
					info.value = player_indices[cand]
					info.notCheckable = 1
					info.disabled = nil
					info.func = XLootMaster_GiveLoot
					UIDropDownMenu_AddButton(info,level)
				end
			end
		end
		
		XLootMaster.BuildRaidMenuRecipients(level)
		XLootMaster.BuildRaidMenuSpecialRolls(level)
		
	end
end

local function XLootMaster_DropdownInit()
	slot = LootFrame.selectedSlot or 0
	info = UIDropDownMenu_CreateInfo()
	
	if UIDROPDOWNMENU_MENU_LEVEL == 1 then
		
		XLootMaster.AddMenuTitle(GIVE_LOOT)
		if ( IsInRaid() ) then
			XLootMaster.BuildRaidMenu(UIDROPDOWNMENU_MENU_LEVEL)
		else
			XLootMaster.BuildPartyMenu(UIDROPDOWNMENU_MENU_LEVEL)
		end
		-- XLootMaster.AddMenuSeparator()
		XLootMaster.BuildRaidMenuSpecialRolls(UIDROPDOWNMENU_MENU_LEVEL)
		
 	elseif UIDROPDOWNMENU_MENU_LEVEL == 2 then

		XLootMaster.BuildRaidMenu(UIDROPDOWNMENU_MENU_LEVEL)

 	end	
end
UIDropDownMenu_Initialize(GroupLootDropDown, XLootMaster_DropdownInit, "MENU")

function XLootMaster.SetBanker(name)
	if not IsInRaid() then return end
	local name = name or UnitExists("target") and (UnitName("target"))
	if UnitInRaid(name) and UnitIsPlayer(name) then
		banker_name = name
	end
end

function XLootMaster.SetDisenchanter(name)
	if not IsInRaid() then return end
	local name = name or UnitExists("target") and (UnitName("target"))
	if UnitInRaid(name) and UnitIsPlayer(name) then
		disenchanter_name = name
	end
end

---------------------------------------------------------------------------
-- AddOn setup and events
---------------------------------------------------------------------------
LibStub('X-Profile'):Setup(addon, 'XLootMaster_Options')
function addon:OnProfileChanged(options)
	opt = options
	addon.opt = opt
end

function addon:OnLoad()
	-- Register events

-- 	LD = LibStub('LibDropdown-1.0')
	LibStub('X-Config'):Setup(addon, 'XLootMasterOptionPanel')

end

-- Slash command for showing options
local function option_handler(msg)
	if msg == 'reset' then
		addon:ResetProfile()
	else
		addon:ShowOptions()
	end
end

SLASH_XLOOTMASTER1 = '/xlm'
SLASH_XLOOTMASTER2 = '/xlootmaster'
SlashCmdList['XLOOTMASTER'] = option_handler

BINDING_HEADER_XLOOTMASTER = addon.NAME
BINDING_NAME_XLM_SETBANKER = L.BINDING_BANKER
BINDING_NAME_XLM_SETDE = L.BINDING_DISENCHANTER

--[[
TODO:
Announce rolls
Announce distribution


Notes:
/run local m,k,n=15,1,"";for i=1,m do n=(k==1) and (i) or (n..","..i) if i==m or k==3 then print(n)n=1 end;k=k<3 and k+1 or 1 end
UnitPopupMenus["LOOT_THRESHOLD"] = { "ITEM_QUALITY2_DESC", "ITEM_QUALITY3_DESC", "ITEM_QUALITY4_DESC", "CANCEL" };
LibDropdown-1.0 supported options
	Color
	Execute
	Groups
	Header
	Input
	Range
	Select
	Toggle
]]
